        .486p
DGROUP group _DATA,_BSS
_TEXT  segment dword public use32 'CODE'
       assume cs:_TEXT, ds:DGROUP, ss:DGROUP

include defs.inc
include func\memory.inc

;--------------------------------------
; Assembler part of Cia
; Fellow Amiga Emulator
; Petter Schau 1997
; additional patches
; Roman Dolejsi
; David Voracek
;--------------------------------------

; -------------------
; Cia register bank  
; -------------------

public readciabyte_
readciabyte_:   push     ecx
                mov     edx,ecx
                and     edx,0a01001h
                cmp     edx,0a00001h            ; Cia a register??
                jne     rbciab
                and     ecx,0f00h               ; Yes! Cia a register
                shr     ecx,6                   ; Get subregister
                call    dword ptr _ciaaread[ecx]
                pop     ecx
                ret

rbciab:         mov     edx,ecx
                and     edx,0a02001h
                cmp     edx,0a00000h
                jne     rciadefault_
                and     ecx,0f00h               ; It is a Cia b register
                shr     ecx,6
                call     dword ptr _ciabread[ecx]
                pop     ecx
                ret

; Take the easy way out..

public readciaword_
readciaword_:   push    ecx
                push    ecx
                call    readciabyte_
                pop     ecx
                shl     edx,8
                push    edx
                inc     ecx
                call    readciabyte_
                pop     ecx
                or      edx,ecx
                pop     ecx
                ret

public readcialong_
readcialong_:   push    ecx
                push    ecx
                call    readciabyte_
                pop     ecx
                inc     ecx
                shl     edx,8
                push    ecx
                push    edx
                call    readciabyte_
                pop     ecx
                or      edx,ecx
                shl     edx,8
                pop     ecx
                inc     ecx
                push    ecx
                push    edx
                call    readciabyte_
                pop     ecx
                or      edx,ecx
                shl     edx,8
                pop     ecx
                inc     ecx
                push    ecx
                push    edx
                call    readciabyte_
                pop     ecx
                or      edx,ecx
                pop     ecx
                pop     ecx
                ret

public writeciabyte_
writeciabyte_:  mov     ecx,dword ptr _wriorgadr
                push    ecx
                and     ecx,0a01001h
                cmp     ecx,0a00001h            ; Cia a register??
                jne     wbciab
                pop     ecx
                and     ecx,0f00h
                shr     ecx,6
                and     edx,0ffh                ; Enforce no extra bits
                call     dword ptr _ciaawrite[ecx]
                ret

wbciab:         pop     ecx
                push    ecx
                and     ecx,0a02001h
                cmp     ecx,0a00000h
                jne     wciadefault_
                pop     ecx
                and     ecx,0f00h
                shr     ecx,6
                and     edx,0ffh                ; Enforce no extra bits
                call     dword ptr _ciabwrite[ecx]
                ret

; Take the easy way out..

public writeciaword_
writeciaword_:  mov     ecx,dword ptr _wriorgadr
                push    ecx
                push    edx
                shr     edx,8
                call    writeciabyte_
                pop     edx
                pop     ecx
                inc     dword ptr _wriorgadr
                jmp     writeciabyte_

public writecialong_
writecialong_:  mov     ecx,dword ptr _wriorgadr
                push    edx
                shr     edx,24
                call    writeciabyte_
                pop     edx
                inc     dword ptr _wriorgadr
                push    edx
                shr     edx,16
                call    writeciabyte_
                pop     edx
                inc     dword ptr _wriorgadr
                push    edx
                shr     edx,8
                call    writeciabyte_
                pop     edx
                inc     dword ptr _wriorgadr
                jmp     writeciabyte_

public wciadefault_
wciadefault_:   pop     ecx
                ret

public rciadefault_
rciadefault_:   pop     ecx
                mov     edx,0ffh  ; Unused cia register seems to be with bits set
                ret


; Scans the timers for the next timeout

public cia_setup_next_event_
cia_setup_next_event_:
                push    ecx
                mov     ecx,-1
                cmp     ecx,dword ptr _ciaataleft
                jbe     cia_sne_1
                mov     ecx,dword ptr _ciaataleft
cia_sne_1:      cmp     ecx,dword ptr _ciaatbleft
                jbe     cia_sne_2
                mov     ecx,dword ptr _ciaatbleft
cia_sne_2:      cmp     ecx,dword ptr _ciabtaleft
                jbe     cia_sne_3
                mov     ecx,dword ptr _ciabtaleft
cia_sne_3:      cmp     ecx,dword ptr _ciabtbleft
                jbe     cia_sne_4
                mov     ecx,dword ptr _ciabtbleft
cia_sne_4:      mov     dword ptr _ciaeventtime,ecx
                pop     ecx
                ret

; Called when a cia event (only timers) has happened

public cia_handle_event_
cia_handle_event_:
                cmp     ecx,dword ptr _ciaataleft       ; Check for timeout in cia a ta
                jb      notimeoutata
                call    ciaa_handle_ta_timeout
notimeoutata:   cmp     ecx,dword ptr _ciaatbleft       ; Check for timeout in cia a tb
                jb      notimeoutatb
                call    ciaa_handle_tb_timeout
notimeoutatb:   cmp     ecx,dword ptr _ciabtaleft       ; Check for timeout in cia b ta
                jb      notimeoutbta
                call    ciab_handle_ta_timeout
notimeoutbta:   cmp     ecx,dword ptr _ciabtbleft       ; Check for timeout in cia b tb
                jb      notimeoutbtb
                call    ciab_handle_tb_timeout
notimeoutbtb:   call    cia_setup_next_event_
                ret




;-------
; CIA A
;-------

; pra, return data about disk drives.
; Return data for the first selected drive, not entirely correct
; When motor is off, return RDY=0 (ID), when motor is on, return
; 1 (Not ready)

public rciaapra_
rciaapra_:      xor     edx,edx  ; 2 fire buttons, pressed
                cmp     byte ptr _fire1A,1
                je      afire
                or      dl,040h
afire:          cmp     byte ptr _fire1B,1
                je      fbutt
                or      dl,080h
fbutt:          cmp     dword ptr _sel,1    ; Check drive 0
                jne     ciaachecksel1
                mov     ecx,0
                jmp     ciaapracheckbits
ciaachecksel1:  cmp     dword ptr _sel+4,1
                jne     ciaachecksel2
                mov     ecx,1
                jmp     ciaapracheckbits
ciaachecksel2:  cmp     dword ptr _sel+8,1
                jne     ciaachecksel3
                mov     ecx,2
                jmp     ciaapracheckbits
ciaachecksel3:  cmp     dword ptr _sel+12,1
                jne     ciaanodriveselected
                mov     ecx,3
ciaapracheckbits:               ; We know the active drive, now check bits
                                ; Check if disk is at track 0
                cmp     dword ptr _track+4*ecx,0
                je      ciaapratrack0     
                or      dl,10h                  ; Set tr0 when track is not 0
ciaapratrack0:                                  ; Check write protect
                cmp     dword ptr _writeprot+4*ecx,1
                je      ciaawriteprot     
                or      dl,8h                  ; Set wprot when not write protected
ciaawriteprot:

; If active = 1, then a disk is recognized as inserted
; That means changed bit is 1 (not changed)
; Active becomes 1 when disk is inserted and a step pulse is
; made
                cmp     dword ptr _active+4*ecx,1
                jne     ciaanodiskindrive
                or      dl,4                   ; Disk inserted 

ciaanodiskindrive:
; Initally not ready, but if disk is active and motor is on, report

                or      dl,20h
                cmp     dword ptr _motor+4*ecx,1
                jne     ciaaidprocedure

                ; Motor is on, if inserted, report ready

                cmp    dword ptr _inserted+4*ecx,1
                jne    ciaanotinserted

        ; Disk is inserted, motor is on, report ready

                and     dl,0dfh
                jmp     blahblah

ciaaidprocedure:
                ; Test if drive is enabled

                cmp     byte ptr [_config_floppy_enabled + 4*ecx],1
                jne     ciaanotinserted

                ; Check how many bits we have read so far
                
                cmp     dword ptr _idcount+4*ecx,32
                je      blahblah
                and     dl,0dfh

; Motor is on but disk not inserted, report not ready (default)
ciaanotinserted:
blahblah:       ret

ciaanodriveselected:
                mov     edx,03ch   ; All lines high
                cmp     byte ptr _fire1B,1
                je      fbutt2
                or      dl,80h
fbutt2:         cmp     byte ptr _fire1A,1
                je      fbutt3
                or      dl,40h
fbutt3:         ret

; Only led is handled here
; 1. Compare new and old value
; 2. Redraw if they're different

public wciaapra_
wciaapra_:              mov     ecx,dword ptr _led
                        and     edx,02h
                        cmp     edx,ecx
                        je      wciaapranoledredraw
                        mov     dword ptr _led,edx
                        xor     ecx,ecx
                        test    edx,2
                        jnz     wciaapraledoff
                        mov     ecx,[_powerledcolor]
wciaapraledoff:         mov     edx,100
                        call    drawled
wciaapranoledredraw:    ret


; Parallell port not implemented

public rciaaprb_
public wciaaprb_
rciaaprb_:              xor     edx,edx
wciaaprb_:              ret

; Hardwired

public rciaaddra_
public wciaaddra_
rciaaddra_:     mov     edx,03h
wciaaddra_:     ret

; Par port direction lines, accept writes and reads.

public rciaaddrb_
public wciaaddrb_
rciaaddrb_:     mov     edx,dword ptr _ciaaddrb
                ret
wciaaddrb_:     mov     dword ptr _ciaaddrb,edx
                ret

; Keyboard serial port data

public rciaasp_
public wciaasp_
rciaasp_:       mov     edx,dword ptr _ciaasp
wciaasp_:       ret


; **********************************************************************************************
; TIMER A IN CIA A
; **********************************************************************************************
	 
; Timer A read counter values:
; ----------------------------
; Simply return the counter values
; Counter values are not always up to date due to the implementation scheme,
; must translate first.
; When timer is stopped, the values are OK.

	
; ------------------- 
; CIA A READ TALO
; -------------------

public rciaatalo_
rciaatalo_:     mov     edx,dword ptr _ciaata
                test    dword ptr _ciaacra,1    ; test if started
                jz      rciaatalo1              ; No, values OK
                mov     edx,dword ptr _ciaataleft  ; Timer started, must translate value first
                call    cia_stabilize_value
rciaatalo1:     and     edx,0ffh
                ret

; ------------------- 
; CIA A READ TAHI
; -------------------

public rciaatahi_
rciaatahi_:     mov     edx,dword ptr _ciaata
                test    dword ptr _ciaacra,1    ; test if started
                jz      rciaatahi1              ; No, values OK
                mov     edx,dword ptr _ciaataleft;  Timer is started, must translate value first
                call    cia_stabilize_value
rciaatahi1:     shr     edx,8
                and     edx,0ffh
                ret

; Timer A write (to latch):
;-----------------
; LO - no side effects
; HI - timer is loaded from latch and Start becomes 0

; -------------------
; CIA A WRITE TALO
; -------------------
; Writes to LO are always latched

public wciaatalo_
wciaatalo_:     mov     byte ptr _ciaatalatch,dl
                ret

; ------------------- 
; CIA A WRITE TAHI
; -------------------
; Write byte to latch, then load the latch into the timer.
; If we decide to load the timer, and oneshot mode is set then start timer
; Hmm, thanks, Bernd, for some info about the CIA.   
; 

public wciaatahi_
wciaatahi_:     mov     byte ptr _ciaatalatch+1,dl  ; Write data to latch
                test    dword ptr _ciaacra,1    ; Timer started?
                jnz     wciaatahistarted          ; Don't load when started
                call    ciaa_stabilize          ; Stabilize ciaa
                mov     edx,dword ptr _ciaatalatch   ; Move latch to timer
                mov     dword ptr _ciaata,edx
                test    dword ptr _ciaacra,8h  ;  IN oneshot mode?
                jz      wciaatahinooneshot
                or      dword ptr _ciaacra,1
wciaatahinooneshot:
                call    ciaa_unstabilize
wciaatahistarted: ; Timer started
                call    cia_setup_next_event_
                ret

; **********************************************************************************************
; END OF TIMER A IN CIA A
; **********************************************************************************************

; **********************************************************************************************
; TIMER B IN CIA A
; **********************************************************************************************

; -------------------
; CIA A READ TBLO
; -------------------

public rciaatblo_
rciaatblo_:      mov     edx,dword ptr [_ciaatb]
                test    dword ptr [_ciaacrb],1    ; test if started
                jz      rciaatblo1              ; No, values OK
                test    dword ptr [_ciaacrb],40h  ; test if timer a attachment
                jnz     rciaatblo1              ; Attached, timer value OK
                mov     edx,dword ptr [_ciaatbleft]; Started and not attached, translate
                call    cia_stabilize_value
rciaatblo1:     and     edx,0ffh
                ret

; ------------------- 
; CIA A READ TBHI
; -------------------

public rciaatbhi_
rciaatbhi_:     mov     edx,dword ptr [_ciaatb]
                test    dword ptr [_ciaacrb],1    ; test if started
                jz      rciaatbhi1              ; No, values OK
                test    dword ptr [_ciaacrb],40h  ; test if timer a attachment
                jnz     rciaatbhi1
                mov     edx,dword ptr [_ciaatbleft]
                call    cia_stabilize_value
rciaatbhi1:     shr     edx,8
                and     edx,0ffh
                ret

; ------------------- 
; CIA A WRITE TBLO
; -------------------
; Writes to LO are always latched

public wciaatblo_
wciaatblo_:     mov     byte ptr [_ciaatblatch],dl
                ret

; ------------------- 
; CIA A WRITE TBHI
; -------------------
; Write byte to latch, then load the latch into the timer.
; If we decide to load the timer, and oneshot mode is set then start timer

public wciaatbhi_
wciaatbhi_:     mov     byte ptr [_ciaatblatch+1],dl
                test    dword ptr [_ciaacrb],1
                jnz     wciaatbhistarted             ; Don't load timer when started
                call    ciaa_stabilize
                mov     edx,dword ptr [_ciaatblatch] ; Load timer with latch
                mov     dword ptr [_ciaatb],edx
                test    dword ptr [_ciaacrb],8h      ; If oneshot is set, start timer
                jz      wciaatbhinooneshot
                or      dword ptr [_ciaacrb],1
wciaatbhinooneshot:
                call    ciaa_unstabilize
wciaatbhistarted:
                call    cia_setup_next_event_
                ret

; **********************************************************************************************
; END OF TIMER B IN CIA A
; **********************************************************************************************
	
; ------------------- 
; CIA A READ EVLO
; -------------------

public rciaaevlo_
rciaaevlo_:     test    dword ptr _ciaaevlatching,1
                jnz     rciaaevlofl
                movzx   edx,byte ptr _ciaaev
                ret
rciaaevlofl:    mov     dword ptr _ciaaevlatching,0
                movzx   edx,byte ptr _ciaaevlatch
                mov     dword ptr _ciaaevlatch,0
                ret

; ------------------- 
; CIA A READ EVMI
; -------------------

public rciaaevmi_
rciaaevmi_:     test    dword ptr _ciaaevlatching,1
                jz      rciaaevmidl
                movzx   edx,byte ptr _ciaaevlatch+1
                ret
rciaaevmidl:    movzx   edx,byte ptr _ciaaev+1
                ret

; ------------------- 
; CIA A READ EVHI
; -------------------

public rciaaevhi_
rciaaevhi_:     mov     edx,dword ptr _ciaaev
                mov     dword ptr _ciaaevlatch,edx
                mov     dword ptr _ciaaevlatching,1
                movzx   edx,byte ptr _ciaaevlatch+2
                ret

; ------------------- 
; CIA A WRITE EVLO
; -------------------

public wciaaevlo_
wciaaevlo_:     test    dword ptr _ciaacrb,80h
                jnz     wciaaevloalarm
                mov     dword ptr _ciaaevwritelatching,0
                mov     byte ptr _ciaaevwritelatch,dl
                mov     edx,dword ptr _ciaaevwritelatch
                mov     dword ptr _ciaaev,edx
                cmp     edx,dword ptr _ciaaevalarm
                jne     ciaaevloex1
                test    dword ptr _ciaacrb,80h
                jz      ciaaevloex1
                or      dword ptr _ciaaicrreq,04h
                test    dword ptr _ciaaicrmsk,04h
                jz      ciaaevloex1
                or      dword ptr _ciaaicrreq,80h
                mov     edx,08008h
                mov     ecx,0dff09ch
                call    wriw
ciaaevloex1:    ret
wciaaevloalarm: mov     dword ptr _ciaaevwritelatching,0
                mov     byte ptr _ciaaevwritelatch,dl
                mov     edx,dword ptr _ciaaevwritelatch
                mov     dword ptr _ciaaevalarm,edx
                ret

; ------------------- 
; CIA A WRITE EVMI
; -------------------

public wciaaevmi_
wciaaevmi_:     test    dword ptr _ciaacrb,80h
                jnz     wciaaevmialarm 
                mov     dword ptr _ciaaevwritelatching,1
                mov     byte ptr _ciaaevwritelatch+1,dl
                ret
wciaaevmialarm: mov     dword ptr _ciaaevwritelatching,1
                mov     byte ptr _ciaaevwritelatch+1,dl
                ret

; ------------------- 
; CIA A WRITE EVHI
; -------------------

public wciaaevhi_
wciaaevhi_:     test    dword ptr _ciaacrb,80h
                jnz     wciaaevhialarm
                mov     dword ptr _ciaaevwritelatching,1
                mov     byte ptr _ciaaevwritelatch+2,dl
                ret
wciaaevhialarm: mov     dword ptr _ciaaevwritelatching,1
                mov     byte ptr _ciaaevwritelatch+2,dl
                ret

; ------------------- 
; CIA A READ ICR
; -------------------

public rciaaicr_
rciaaicr_:      mov     edx,dword ptr _ciaaicrreq
                mov     dword ptr _ciaaicrreq,0
                ret

; --------------------- 
; CIA A WRITE ICR (MSK)
; ---------------------

public wciaaicr_
wciaaicr_:      or      dl,dl
                js      wciaaicrset
                and     edx,01fh
                not     edx
                and     dword ptr _ciaaicrmsk,edx
                ret
wciaaicrset:    and     edx,1fh
                or      dword ptr _ciaaicrmsk,edx
                mov     dl,byte ptr _ciaaicrmsk
                mov     dh,byte ptr _ciaaicrreq
                and     dl,dh
                jz      wciaaicrnoirq
                mov     edx,08008h
                mov     ecx,0dff09ch
                call    wriw
wciaaicrnoirq:  ret

; **********************************************************************************************
; CONTROL REGISTERS FOR TIMERS, CIA A
; **********************************************************************************************
	
; ------------------- 
; CIA A READ CRA
; -------------------

public rciaacra_
rciaacra_:      mov     edx,dword ptr [_ciaacra]
                ret

; ------------------- 
; CIA A WRITE CRA
; -------------------

public wciaacra_
wciaacra_:      call    ciaa_stabilize
                test    edx,10h                    
                jz      wciaacra1
                mov     ecx,dword ptr [_ciaatalatch]  ; Load bit is set, reload timer
                mov     dword ptr [_ciaata],ecx
                and     edx,0efh                    ; Clear load bit
wciaacra1:      mov     dword ptr [_ciaacra],edx     ; Store new cra
                call    ciaa_unstabilize
                call    cia_setup_next_event_
                ret 

; ------------------- 
; CIA A READ CRB
; -------------------

public rciaacrb_
rciaacrb_:      mov     edx,dword ptr [_ciaacrb]
                ret

; ------------------- 
; CIA A WRITE CRB
; -------------------

public wciaacrb_
wciaacrb_:      call    ciaa_stabilize
                test    edx,10h
                jz      wciaacrb1
                mov     ecx,dword ptr [_ciaatblatch] ; Load bit is set, reload timer
                mov     dword ptr [_ciaatb],ecx
                and     edx,0efh                     ; Clear load bit
wciaacrb1:      mov     dword ptr [_ciaacrb],edx     ; Store new crb value
                call    ciaa_unstabilize
                call    cia_setup_next_event_
                ret


; **********************************************************************************************
; END OF CONTROL REGISTERS FOR TIMERS, CIA A
; **********************************************************************************************
	

; **********************************************************************************************
; TIMEOUT HANDLERS FOR TIMERS, CIA A
; **********************************************************************************************

; Only called when timer is running.
; Reload counter from latch
; If timer b is in mode 10, decrease tb counter
; If counter is not in one-shot mode

ciaa_handle_ta_timeout:
                push    edx
                push    ecx
                mov     edx,dword ptr _ciaatalatch      ; Reload from latch
                mov     dword ptr _ciaata,edx
                test    dword ptr _ciaacrb,40h          ; Handle timer b if attached
                jz      ciaa_hta2                       ; Jump if no timer b attachment
                test    dword ptr _ciaacrb,1            ; Is timer b started?
                jz      ciaa_hta2
                and     dword ptr _ciaatb,0ffffh
                dec     dword ptr _ciaatb
                jnz     ciaa_hta2                       ; If tb != 0 don't do anything
                call    ciaa_handle_tb_timeout          ; Timer b handles itself
ciaa_hta2:	test	dword ptr _ciaacra,8
                jnz     ciaa_hta1               ; One shot mode then jump
                mov     edx,dword ptr _ciaata   ; Here: Continuous mode
                call    cia_unstabilize_value   ; Recalculate timer
                mov     dword ptr _ciaataleft,edx
                jmp     ciaa_hta3
ciaa_hta1:      and     dword ptr _ciaacra,0feh  ; Stop timer after timeout in one shot mode
                mov     dword ptr _ciaataleft,-1
ciaa_hta3:      or      dword ptr _ciaaicrreq,01h 
                test    dword ptr _ciaaicrmsk,1 ; Now see if we must raise an interrupt
                jz      ciaa_hta4               ; Jump if not
                or      dword ptr _ciaaicrreq,80h 
                mov     ecx,0dff09ch
                mov     edx,08008h
                call    wriw
ciaa_hta4:	pop	ecx
                pop     edx
                ret

; Only called when timer is running.
; Reload counter from latch
; If counter is not in one-shot mode or attached to timer a
; Call ciaa_recalculate_timer_b to translate the new value
ciaa_handle_tb_timeout:
                push    edx
                push    ecx
                mov     edx,dword ptr _ciaatblatch ; Reload from latch
                mov     dword ptr _ciaatb,edx
                test    dword ptr _ciaacrb,8
                jnz     ciaa_htb1               ; One shot mode then jump
                test    dword ptr _ciaacrb,40h
                jnz      ciaa_htb3                       ; Jump if timer a attachment
                mov     edx,dword ptr _ciaatb    ; Here, continuous mode
                call    cia_unstabilize_value; Else recalculate timer
                mov     dword ptr _ciaatbleft,edx
                jmp     ciaa_htb3
ciaa_htb1:      and     dword ptr _ciaacrb,0feh  ; Stop timer after timeout in one shot mode
                mov     dword ptr _ciaatbleft,-1
ciaa_htb3:      or      dword ptr _ciaaicrreq,02h 
                test    dword ptr _ciaaicrmsk,2 ; Now see if we must raise an interrupt
                jz      ciaa_htb4               ; Jump if not
                or      dword ptr _ciaaicrreq,80h 
                mov     ecx,0dff09ch
                mov     edx,08008h
                call    wriw
ciaa_htb4:	pop	ecx
                pop     edx
                ret

; **********************************************************************************************
; END OF TIMEOUT HANDLERS FOR TIMERS, CIA A
; **********************************************************************************************


; **********************************************************************************************
; SOME ROUTINES TO HANDLE THE STATE OF TIMERS, CIA A
; **********************************************************************************************

; Parameter in edx, out in edx
cia_unstabilize_value:		
                push    ecx
                mov     ecx,edx
                shl     edx,2
                add     edx,ecx
                add     edx,dword ptr [_curcycle]
                pop     ecx
                ret

; Parameter in edx, out in edx too
cia_stabilize_value:
                push    eax
                push    ecx
                mov     eax,edx 
                sub     eax,dword ptr [_curcycle]
                mov     ecx,5
                xor     edx,edx
                div     ecx
                mov     edx,eax
                and     edx,0ffffh
                pop     ecx
                pop     eax
                ret

; If timer is stopped, there's no timer to translate
; Else translate timer value
ciaata_stabilize:
                test    dword ptr [_ciaacra],1
                jz      ciaata_stab1
                mov     edx,dword ptr [_ciaataleft]
                call    cia_stabilize_value
                mov     dword ptr [_ciaata],edx
ciaata_stab1:   mov     dword ptr [_ciaataleft],-1
                ret 

; If timer is stopped, there's no timer to translate
; If timer is A attached, there's no timer to translate
; Else translate timer value
ciaatb_stabilize:
                test    dword ptr [_ciaacrb],1
                jz      ciaatb_stab1
                test    dword ptr [_ciaacrb],40h
                jnz     ciaatb_stab1
                mov     edx,dword ptr [_ciaatbleft]
                call    cia_stabilize_value
                mov     dword ptr [_ciaatb],edx
ciaatb_stab1:   mov     dword ptr [_ciaatbleft],-1
                ret

ciaa_stabilize:
                push    edx
                call    ciaata_stabilize
                call    ciaatb_stabilize
                pop     edx
                ret

ciaata_unstabilize:
                test    dword ptr [_ciaacra],1
                jz      ciaata_unstab1
                mov     edx,dword ptr [_ciaata]
                call    cia_unstabilize_value
                mov     dword ptr [_ciaataleft],edx
ciaata_unstab1:	ret

ciaatb_unstabilize:
                test    dword ptr [_ciaacrb],1
                jz      ciaatb_unstab1
                test    dword ptr [_ciaacrb],40h      ; Attached, don't translate
                jnz     ciaatb_unstab1
                mov     edx,dword ptr [_ciaatb]
                call    cia_unstabilize_value
                mov     dword ptr [_ciaatbleft],edx
ciaatb_unstab1: ret

ciaa_unstabilize:
                push    edx
                call    ciaata_unstabilize
                call    ciaatb_unstabilize
                pop     edx
                ret
	
	
; **********************************************************************************************
; END OF SOME ROUTINES TO HANDLE THE STATE OF TIMERS, CIA A
; **********************************************************************************************

	
public rcianothing_
public wcianothing_
rcianothing_:   mov     edx,0ffh
wcianothing_:   ret
		

;-------
; CIA B
;-------

; pra is parallell port control data, 
; ignore writes, and always return 0dch

public rciabpra_
public wciabpra_
rciabpra_:      mov     edx,0dch
wciabpra_:      ret

; prb, Ignore reads because all lines are hardwired? for input 
; return zero, but this might not be right

public rciabprb_
rciabprb_:      mov     edx,dword ptr _ciabprb
                ret

; This procedure is utterly stupid, should be reprogrammed

public wciabprb_
wciabprb_:
; Zero out select variables
                mov     dword ptr _ciabprb,edx
                mov     dword ptr _sel,0
                mov     dword ptr _sel+4,0
                mov     dword ptr _sel+8,0
                mov     dword ptr _sel+12,0
                mov     cl,byte ptr [_motor]
                mov     byte ptr [_oldmotorstate],cl
                mov     cl,byte ptr [_motor+4]
                mov     byte ptr [_oldmotorstate+1],cl
                mov     cl,byte ptr [_motor+8]
                mov     byte ptr [_oldmotorstate+2],cl
                mov     cl,byte ptr [_motor+12]
                mov     byte ptr [_oldmotorstate+3],cl

                test    edx,08h   ; Sel0
                jnz     prachecksel1
                mov     dword ptr _sel,1    ; Set select

; If motor goes off and was on, reset idcounter

                test    dword ptr _motor,1
                jz      sel0noidnow
                test    edx,80h        ; Motor was on, check if it is turned off
                jz      sel0noidnow
                mov     dword ptr _idcount,0
sel0noidnow:    mov     dword ptr _motor,0  ; Zero out motor,
                test    edx,80h                 ; MTR is active low
                jnz     sel0side
                mov     dword ptr _motor,1  ; Set motor again
sel0side:       mov     dword ptr _side,0   ; Zero out side
                test    edx,4
                jnz     sel0dir                 ; SID is active low
                mov     dword ptr _side,1   ; Set side
sel0dir:        mov     dword ptr _dir,1      ; Zero out dir
                test    edx,2
                jnz     sel0step                ; 1 is towards track 0
                mov     dword ptr _dir,0    ; Set dir
sel0step:       test    edx,1
                jnz     sel0nostep
                cmp     dword ptr _inserted,1
                jne     sel0nochng
                mov     dword ptr _active,1   ; Step, if disk is inserted, active->1
sel0nochng:     test    edx,2
                jz      sel0stepincrease
                cmp     dword ptr _track,0
                je      sel0stepdecreasenot
                dec     dword ptr _track
sel0stepdecreasenot:
                jmp     sel0nostep
sel0stepincrease:
                cmp     dword ptr _track,79
                je      sel0stepincreasenot
                inc     dword ptr _track
sel0ignore:
sel0stepincreasenot:
sel0nostep:

; Drive 1
prachecksel1:
        
        test    edx,00000010h   ; Sel1
        jnz     prachecksel2

        mov     dword ptr _sel+4,1    ; Set select

; If motor goes off and was on, reset idcounter

        test    dword ptr _motor+4,1
        jz      sel1noidnow
        test    edx,80h        ; Motor was on, check if it is turned off
        jz      sel1noidnow
        mov     dword ptr _idcount+4,0
sel1noidnow:
        mov     dword ptr _motor+4,0  ; Zero out motor
        test    edx,80h
        jnz     sel1side
        mov     dword ptr _motor+4,1  ; Set motor again
sel1side:
        mov     dword ptr _side+4,0   ; Zero out side
        test    edx,4
        jnz     sel1dir
        mov     dword ptr _side+4,1   ; Set side
sel1dir:
        mov     dword ptr _dir+4,1      ; Zero out dir
        test    edx,2
        jnz     sel1step
        mov     dword ptr _dir+4,0    ; Set dir
sel1step:
        test    edx,1
        jnz     sel1nostep

        cmp     dword ptr _inserted+4,1
        jne     sel1nochng
        mov     dword ptr _active+4,1   ; Step, if disk is inserted, active->1
sel1nochng:

        test    edx,2
        jz      sel1stepincrease
        cmp     dword ptr _track+4,0
        je      sel1stepdecreasenot
        dec     dword ptr _track+4
sel1stepdecreasenot:
        jmp     sel1nostep
sel1stepincrease:
        cmp     dword ptr _track+4,79
        je      sel1stepincreasenot
        inc     dword ptr _track+4
sel1ignore:
sel1stepincreasenot:
sel1nostep:
; Drive 2
prachecksel2:
        test    edx,20h   ; Sel2
        jnz     prachecksel3

        mov     dword ptr _sel+8,1    ; Set select

; If motor goes off and was on, reset idcounter

        test    dword ptr _motor+8,1
        jz      sel2noidnow
        test    edx,80h        ; Motor was on, check if it is turned off
        jz      sel2noidnow
        mov     dword ptr _idcount+8,0
sel2noidnow:
        mov     dword ptr _motor+8,0  ; Zero out motor
        test    edx,80h
        jnz     sel2side
        mov     dword ptr _motor+8,1  ; Set motor again
sel2side:
        mov     dword ptr _side+8,0   ; Zero out side
        test    edx,4
        jnz     sel2dir
        mov     dword ptr _side+8,1   ; Set side
sel2dir:
        mov     dword ptr _dir+8,1      ; Zero out dir
        test    edx,2
        jnz     sel2step
        mov     dword ptr _dir+8,0    ; Set dir
sel2step:
        test    edx,1
        jnz     sel2nostep

        cmp     dword ptr _inserted+8,1
        jne     sel2nochng
        mov     dword ptr _active+8,1   ; Step, if disk is inserted, active->1
sel2nochng:

        test    edx,2
        jz      sel2stepincrease
        cmp     dword ptr _track+8,0
        je      sel2stepdecreasenot
        dec     dword ptr _track+8
sel2stepdecreasenot:
        jmp     sel2nostep
sel2stepincrease:
        cmp     dword ptr _track+8,79
        je      sel2stepincreasenot
        inc     dword ptr _track+8
sel2ignore:
sel2stepincreasenot:
sel2nostep:

; Drive 3
prachecksel3:
        test    edx,40h   ; Sel3
        jnz     prachecksel4

        mov     dword ptr _sel+12,1    ; Set select

; If motor goes off and was on, reset idcounter

        test    dword ptr _motor+12,1
        jz      sel3noidnow
        test    edx,80h        ; Motor was on, check if it is turned off
        jz      sel3noidnow
        mov     dword ptr _idcount+12,0
sel3noidnow:
        mov     dword ptr _motor+12,0  ; Zero out motor
        test    edx,80h
        jnz     sel3side
        mov     dword ptr _motor+12,1  ; Set motor again
sel3side:
        mov     dword ptr _side+12,0   ; Zero out side
        test    edx,4
        jnz     sel3dir
        mov     dword ptr _side+12,1   ; Set side
sel3dir:
        mov     dword ptr _dir+12,1      ; Zero out dir
        test    edx,2
        jnz     sel3step
        mov     dword ptr _dir+12,0    ; Set dir
sel3step:
        test    edx,1
        jnz     sel3nostep

        cmp     dword ptr _inserted+12,1
        jne     sel3nochng
        mov     dword ptr _active+12,1   ; Step, if disk is inserted, active->1
sel3nochng:

        test    edx,2
        jz      sel3stepincrease
        cmp     dword ptr _track+12,0
        je      sel3stepdecreasenot
        dec     dword ptr _track+12
sel3stepdecreasenot:
        jmp     sel3nostep
sel3stepincrease:
        cmp     dword ptr _track+12,79
        je      sel3stepincreasenot
        inc     dword ptr _track+12
sel3ignore:
sel3stepincreasenot:
sel3nostep:
prachecksel4:


        push    ebx
        push    eax
        xor     ebx,ebx
        mov     edx,150
wciabprbmotorledloop:
        xor     ecx,ecx
        test    byte ptr [_motor+4*ebx],1
        jz      wciabprbmotorled
        mov     ecx,[_drivemotorcolor]
wciabprbmotorled:
        cmp     ecx, [_drivemotorcolor]
        jne     ledre1
        mov     eax, [_config_graphics_mode]
        mov     eax, [eax * 4 + _config_graphics_moderesy]
        cmp     eax, 200
        je      ledre2
        cmp     eax, 240
        je      ledre2
ledre1: mov     al,byte ptr [_motor+4*ebx]
        cmp     al,byte ptr [_oldmotorstate+ebx]
        je      noledre
        call    drawled
        jmp     noledre
ledre2: cmp     ecx, [_drivemotorcolor]
        jne     noledre
        call    drawled
noledre:inc     ebx
        add     edx,50
        cmp     ebx,4
        jne     wciabprbmotorledloop
        pop     eax
        pop     ebx
        ret

; ddra is hardwired to 0c0h

public rciabddra_
public wciabddra_
rciabddra_:
        mov     edx,0c0h
wciabddra_:
        ret
; ddrb is hardwired to 0ffh
public rciabddrb_
public wciabddrb_
rciabddrb_:
        mov     edx,0ffh
wciabddrb_:
        ret

; Serial port, not implemented
public rciabsp_
public wciabsp_
rciabsp_:
	xor	edx,edx
wciabsp_:
        ret


; **********************************************************************************************
; TIMER A IN CIA B
; **********************************************************************************************
	 
; Timer A read counter values:
; ----------------------------
; Simply return the counter values
; Counter values is not always up to date due to the implementation scheme, must translate first
; When timer is stopped, the values are OK.

	
; ------------------- 
; CIA B READ TALO
; -------------------

public rciabtalo_
rciabtalo_:
        test    dword ptr _ciabcra,1    ; test if started
        jz      rciabtalo1              ; No, values OK
        mov     edx,dword ptr _ciabtaleft  ; Timer started, must translate value first
        call    cia_stabilize_value
        mov     dword ptr _ciabta,edx
rciabtalo1:
        movzx   edx,byte ptr _ciabta  ; Here the timer value is valid
        ret

; ------------------- 
; CIA B READ TAHI
; -------------------

public rciabtahi_
rciabtahi_:
        test    dword ptr _ciabcra,1    ; test if started
        jz      rciabtahi1              ; No, values OK
        mov     edx,dword ptr _ciabtaleft;  Timer is started, must translate value first
        call    cia_stabilize_value
        mov     dword ptr _ciabta,edx
rciabtahi1:
        movzx   edx,byte ptr _ciabta+1  ; Here the timer value is valid
        ret

	

;
; Timer A write (to latch):
;-----------------
; LO - no side effects
; HI - timer is loaded from latch and Start becomes 0

; ------------------- 
; CIA B WRITE TALO
; -------------------
; Writes to LO are always latched

public wciabtalo_
wciabtalo_:
        mov     byte ptr _ciabtalatch,dl
        ret

; ------------------- 
; CIA B WRITE TAHI
; -------------------
; Write byte to latch, then load the latch into the timer.
; Seems OK

public wciabtahi_
wciabtahi_:
        mov     byte ptr _ciabtalatch+1,dl
        call    ciab_stabilize
        test    dword ptr _ciabcra,1    ; Timer started?
	jnz	wciabtastarted
        mov     edx,dword ptr _ciabtalatch
        mov     dword ptr _ciabta,edx
        test    dword ptr _ciabcra,8h  ;  IN oneshot mode?
        jz      wciabtastarted
        or      dword ptr _ciabcra,1
wciabtastarted: ; Timer started
        call    ciab_unstabilize
        call    cia_setup_next_event_
        ret


	
; **********************************************************************************************
; END OF TIMER A IN CIA B
; **********************************************************************************************
		
	
	

; **********************************************************************************************
; TIMER B IN CIA B
; **********************************************************************************************
; Timer B read counter value:
; ---------------------------
; Simply return the counter values
; Will not return valid values in CNT mode

public rciabtblo_
rciabtblo_:
        test    dword ptr _ciabcrb,1    ; test if started
        jz      rciabtblo1              ; No, values OK
        test    dword ptr _ciabcrb,40h  ; test if timer a attachment
        jnz     rciabtblo1              ; Attached, timer value OK
        mov     edx,dword ptr _ciabtbleft; Started and not attached, translate
        call    cia_stabilize_value
        mov     dword ptr _ciabtb,edx
rciabtblo1:
        movzx   edx,byte ptr _ciabtb
        ret

public rciabtbhi_
rciabtbhi_:
        test    dword ptr _ciabcrb,1    ; test if started
        jz      rciabtbhi1              ; No, values OK
        test    dword ptr _ciabcrb,40h  ; test if timer a attachment
        jnz     rciabtbhi1
        mov     edx,dword ptr _ciabtbleft
        call    cia_stabilize_value
        mov     dword ptr _ciabtb,edx
rciabtbhi1:
        movzx   edx,byte ptr _ciabtb+1
        ret



; Timer B write (to latch):
;-----------------
; LO - no side effects
; HI - timer is loaded from latch and Start becomes 0
;      Is this right, UAE starts timer when in oneshot mode?

public wciabtblo_
wciabtblo_:
        mov     byte ptr _ciabtblatch,dl
        ret

public wciabtbhi_
wciabtbhi_:
        mov     byte ptr _ciabtblatch+1,dl
        call    ciab_stabilize
        mov     edx,dword ptr _ciabtblatch
        and     edx,0ffffh
        mov     dword ptr _ciabtb,edx
        test    dword ptr _ciabcrb,8h
        jz      wciabtbend
        or      dword ptr _ciabcrb,1
wciabtbend:
        call    ciab_unstabilize
        call    cia_setup_next_event_
        ret


; **********************************************************************************************
; END OF TIMER B IN CIA B
; **********************************************************************************************
	
	
	

public rciabevlo_
rciabevlo_:
        test    dword ptr _ciabevlatching,1
        jnz     rciabevlofl
        movzx   edx,byte ptr _ciabev
        ret
rciabevlofl:
        mov     dword ptr _ciabevlatching,0
        movzx   edx,byte ptr _ciabevlatch
        mov     dword ptr _ciabevlatch,0
        ret

public rciabevmi_
rciabevmi_:
        test    dword ptr _ciabevlatching,1
        jz      rciabevmidl
        movzx   edx,byte ptr _ciabevlatch+1
        ret
rciabevmidl:
        mov     edx,dword ptr _ciabev
        mov     dword ptr _ciabevlatch,edx
        mov     dword ptr _ciabevlatching,1
        movzx   edx,byte ptr _ciabevlatch+1
        ret

public rciabevhi_
rciabevhi_:
        mov     edx,dword ptr _ciabev
        mov     dword ptr _ciabevlatch,edx
        mov     dword ptr _ciabevlatching,1
        movzx   edx,byte ptr _ciabevlatch+2
        ret

public wciabevlo_
wciabevlo_:
        test    dword ptr _ciabcrb,80h
        jnz     wciabevloalarm
        mov     dword ptr _ciabevwritelatching,0
        mov     byte ptr _ciabevwritelatch,dl
        mov     edx,dword ptr _ciabevwritelatch
        mov     dword ptr _ciabev,edx
        ret
wciabevloalarm:
        mov     dword ptr _ciabevwritelatching,0
        mov     byte ptr _ciabevwritelatch,dl
        mov     edx,dword ptr _ciabevwritelatch
        mov     dword ptr _ciabevalarm,edx
        ret

public wciabevmi_
wciabevmi_:
        test    dword ptr _ciabcrb,80h
        jnz     wciabevmialarm
        mov     dword ptr _ciabevwritelatching,1
        mov     byte ptr _ciabevwritelatch+1,dl
        ret
wciabevmialarm:
        mov     dword ptr _ciabevwritelatching,1
        mov     byte ptr _ciabevwritelatch+1,dl
        ret

public wciabevhi_
wciabevhi_:
        test    dword ptr _ciabcrb,80h
        jnz     wciabevhialarm
        mov     dword ptr _ciabevwritelatching,1
        mov     byte ptr _ciabevwritelatch+2,dl
        ret
wciabevhialarm:
        mov     dword ptr _ciabevwritelatching,1
        mov     byte ptr _ciabevwritelatch+2,dl
        ret




public rciabicr_
rciabicr_:
        movzx   edx,byte ptr _ciabicrreq
        mov     byte ptr _ciabicrreq,0
        ret

public wciabicr_
wciabicr_:
        or      dl,dl
        js      wciabicrset
        not     edx
        and     dword ptr _ciabicrmsk,edx
        ret
wciabicrset:
        or      dword ptr _ciabicrmsk,edx
        mov     dl,byte ptr _ciabicrmsk
        mov     dh,byte ptr _ciabicrreq
        and     dl,dh
        and     dl,1fh
        jz      wciabicrnoirq
        mov     edx,0a000h
        mov     ecx,0dff09ch
        call    wriw
wciabicrnoirq:
        ret



; **********************************************************************************************
; CONTROL REGISTERS FOR TIMERS, CIA B
; **********************************************************************************************
	

public rciabcra_
rciabcra_:
        mov     edx,dword ptr _ciabcra
        ret

public wciabcra_
wciabcra_:
	push	edx
	call	ciab_stabilize
	pop	edx
	test	edx,10h
	jz	wciabcra1
	mov	ecx,dword ptr _ciabtalatch
	mov	dword ptr _ciabta,ecx
	and	edx,0efh
wciabcra1:	
	mov	dword ptr _ciabcra,edx
	call	ciab_unstabilize
        call    cia_setup_next_event_
	ret	

public rciabcrb_
rciabcrb_:
        mov     edx,dword ptr _ciabcrb
        ret

public wciabcrb_
wciabcrb_:
	push	edx
	call	ciab_stabilize
	pop	edx
	test	edx,10h
	jz	wciabcrb1
	mov	ecx,dword ptr _ciabtblatch
	mov	dword ptr _ciabtb,ecx
	and	edx,0efh
wciabcrb1:	
	mov	dword ptr _ciabcrb,edx
	call	ciab_unstabilize
        call    cia_setup_next_event_
        ret


; **********************************************************************************************
; END OF CONTROL REGISTERS FOR TIMERS, CIA B
; **********************************************************************************************
	
; **********************************************************************************************
; TIMEOUT HANDLERS FOR TIMERS, CIA B
; **********************************************************************************************

; Only called when timer is running.
; Reload counter from latch
; If timer b is in mode 10, decrease tb counter
; If counter is not in one-shot mode
ciab_handle_ta_timeout:
	push	edx
	push	ecx
        mov     edx,dword ptr _ciabtalatch      ; Reload from latch
        mov     dword ptr _ciabta,edx
        test    dword ptr _ciabcrb,40h          ; Handle timer b if attached
        jz      ciab_hta2                       ; Jump if no timer b attachment
;        test    dword ptr _ciabcrb,20h          ; CNT ignored
;        jnz     ciab_hta2                       ; Jump if inmode = 11 (ignored)
        test    dword ptr _ciabcrb,1            ; Is timer b started?
        jz      ciab_hta2
        and     dword ptr _ciabtb,0ffffh
        dec     dword ptr _ciabtb
        jnz     ciab_hta2                       ; If tb != 0 don't do anything
        call    ciab_handle_tb_timeout          ; Timer b handles itself
ciab_hta2: 
        test    dword ptr _ciabcra,8
        jnz     ciab_hta1               ; One shot mode then jump
        mov     edx,dword ptr _ciabta   ; Here: Continuous mode
        call    cia_unstabilize_value   ; Recalculate timer
        mov     dword ptr _ciabtaleft,edx
        jmp     ciab_hta3
ciab_hta1:
        and     dword ptr _ciabcra,0feh  ; Stop timer after timeout in one shot mode
        mov     dword ptr _ciabtaleft,-1
ciab_hta3:
        or      byte ptr _ciabicrreq,01h
        test    byte ptr _ciabicrmsk,1 ; Now see if we must raise an interrupt
        jz      ciab_hta4               ; Jump if not
        or      byte ptr _ciabicrreq,80h 
        mov     ecx,0dff09ch
        mov     edx,0A000h
	call	wriw
ciab_hta4:
	pop	ecx
	pop	edx
	ret

; Only called when timer is running.
; Reload counter from latch
; If counter is not in one-shot mode or attached to timer a
; Call ciab_recalculate_timer_b to translate the new value
ciab_handle_tb_timeout:
	push	edx
	push	ecx
        mov     edx,dword ptr _ciabtblatch ; Reload from latch
        mov     dword ptr _ciabtb,edx
        test    dword ptr _ciabcrb,8
        jnz     ciab_htb1               ; One shot mode then jump
        test    dword ptr _ciabcrb,40h
        jnz      ciab_htb3                       ; Jump if timer a attachment
;        test    dword ptr _ciabcrb,20h          ; CNT ignored
;        jz      ciab_htb3                       ; Jump if inmode = 10
        mov     edx,dword ptr _ciabtb    ; Here, continuous mode
        call    cia_unstabilize_value; Else recalculate timer
        mov     dword ptr _ciabtbleft,edx
        jmp     ciab_htb3
ciab_htb1:
        and     dword ptr _ciabcrb,0feh  ; Stop timer after timeout in one shot mode
        mov     dword ptr _ciabtbleft,-1
ciab_htb3:
        or      dword ptr _ciabicrreq,02h 
        test    dword ptr _ciabicrmsk,2 ; Now see if we must raise an interrupt
        jz      ciab_htb4               ; Jump if not
        or      dword ptr _ciabicrreq,80h 
        mov     ecx,0dff09ch
        mov     edx,0A000h
	call	wriw
ciab_htb4:
	pop	ecx
	pop	edx
	ret


; **********************************************************************************************
; SOME ROUTINES TO HANDLE THE STATE OF TIMERS, CIA B
; **********************************************************************************************


ciabta_stabilize:	
	test	dword ptr _ciabcra,1
	jz	ciabta_stab1
	mov	edx,dword ptr _ciabtaleft
	call	cia_stabilize_value
	mov	dword ptr _ciabta,edx
ciabta_stab1:	
	mov	dword ptr _ciabtaleft,-1
	ret	

ciabtb_stabilize:
	test	dword ptr _ciabcrb,1
	jz	ciabtb_stab1
	test	dword ptr _ciabcrb,20h
	jnz	ciabtb_stab1
	mov	edx,dword ptr _ciabtbleft
	call	cia_stabilize_value
	mov	dword ptr _ciabtb,edx
ciabtb_stab1:
	mov	dword ptr _ciabtbleft,-1
	ret

ciab_stabilize: 
        push    edx
        call	ciabta_stabilize
	call	ciabtb_stabilize
        pop     edx
        ret

ciabta_unstabilize:
	test	dword ptr _ciabcra,1
	jz	ciabta_unstab1
	test	dword ptr _ciabcra,20h
	jnz	ciabta_unstab1
	mov	edx,dword ptr _ciabta
	call	cia_unstabilize_value
	mov	dword ptr _ciabtaleft,edx
ciabta_unstab1:
	ret

ciabtb_unstabilize:
	test	dword ptr _ciabcrb,1
	jz	ciabtb_unstab1
	test	dword ptr _ciabcrb,20h
	jnz	ciabtb_unstab1
	test	dword ptr _ciabcrb,40h
	jnz	ciabtb_unstab1
	mov	edx,dword ptr _ciabtb
        call    cia_unstabilize_value
	mov	dword ptr _ciabtbleft,edx
ciabtb_unstab1:
	ret

ciab_unstabilize:
        call    ciabta_unstabilize
        call    ciabtb_unstabilize
	ret


; **********************************************************************************************
; END OF SOME ROUTINES TO HANDLE THE STATE OF TIMERS, CIA B
; **********************************************************************************************

; Draw LED symbols

; edx - offset in line (xpos*2)
; ecx - color

drawled:        cmp     byte [_config_enableleds], 0
                je      drawleddont
                test    byte [_config_enableleds], 0c0h
                jnz     drawledlpt
                cmp     byte [_config_graphics_mode],5
                ja      drawleddont
drawledscr:     push    eax
                push    ebx
                mov     eax,[_configdoublebuffer]
drawledloop:    push    eax
                push    edx
                add     edx,[_framebuffer]
                add     edx,[_bufferoffsets+4*eax]
                mov     eax, [_config_graphics_mode]
                mov     eax, [eax * 4 + _config_graphics_moderesx]
                add     edx, eax
                add     edx, eax
                add     edx,_scanlineadd
                mov     ebx,2
drawledloop2:   mov     [edx],ecx
                mov     4[edx],ecx
                mov     8[edx],ecx
                mov     12[edx],ecx
                mov     16[edx],ecx
                mov     20[edx],ecx
                mov     24[edx],ecx
                mov     28[edx],ecx
                add     edx,[_scanlineadd]
drawledloop2e:  add     edx, eax
                add     edx, eax
                dec     ebx
                jnz     drawledloop2
                pop     edx
                pop     eax
                dec     eax
                jns     drawledloop
                pop     ebx
                pop     eax
drawleddont:    ret

drawledlpt:     push    eax
                push    edx
                test    byte [_config_enableleds], 10h
                jnz     drawledlptmirr
                mov     ah, 80h
                cmp     edx, 100
                je      drawledlpt0
                mov     ah, 1
                cmp     edx, 150
                je      drawledlpt0
                mov     ah, 2
                cmp     edx, 200
                je      drawledlpt0
                mov     ah, 4
                cmp     edx, 250
                je      drawledlpt0
                mov     ah, 8
drawledlpt0:    mov     al, ah
                not     al
                test    ecx, ecx
                jnz     drawledlpt1
                mov     ah, 0
drawledlpt1:    and     al, [_ledbyte]
                or      al, ah
                mov     [_ledbyte], al
                pop     edx
                pop     eax

public  drawledlpt_
drawledlpt_:    push    eax
                push    edx
                mov     al, [_ledbyte]
                test    byte [_config_enableleds], 80h         ; keyboard output ?
                jnz     drawledkbd
                mov     edx, [_ledlpt]
                out     dx, al
                pop     edx
                pop     eax
                ret
drawledlptmirr: mov     ah, 1
                cmp     edx, 100
                je      drawledlpt0
                mov     ah, 80h
                cmp     edx, 150
                je      drawledlpt0
                mov     ah, 40h
                cmp     edx, 200
                je      drawledlpt0
                mov     ah, 20h
                cmp     edx, 250
                je      drawledlpt0
                mov     ah, 10h
                jmp     short drawledlpt0

drawledkbd:     rol     al, 1
                and     al, 0fh
                movzx   edx, al
                mov     eax, [_config_enableleds]
                dec     al
                and     eax, 3
                shl     eax, 5
                add     edx, eax
                mov     dl, [drawledkbd_tab + edx]
                mov     al, byte ptr [_kbdleds]
                shl     eax, 3
                add     edx, eax
                push    ecx
                mov     ecx, 65536
drawledkbd_w1:  in      al, 61h
                dec     ecx
                jc      drawledkbd_w11
                and     al, 2
                jnz     drawledkbd_w1
drawledkbd_w11: mov     al, 0edh
                out     60h, al
                mov     ecx, 65536
drawledkbd_w2:  in      al, 61h
                dec     ecx
                jc      drawledkbd_w21
                and     al, 2
                jnz     drawledkbd_w2
drawledkbd_w21: mov     al, [drawledkbd_kbd + edx]
                out     60h, al
                pop     ecx
                pop     edx
                pop     eax
                ret

public drawoldkbd_
drawoldkbd_:    push    eax
                push    ecx
                mov     ecx, 65536
drawoldkbd_w1:  in      al, 61h
                dec     ecx
                jc      drawoldkbd_w11
                and     al, 2
                jnz     drawoldkbd_w1
drawoldkbd_w11: mov     al, 0edh
                out     60h, al
                mov     ecx, 65536
drawoldkbd_w2:  in      al, 61h
                dec     ecx
                jc      drawoldkbd_w21
                and     al, 2
                jnz     drawoldkbd_w2
drawoldkbd_w21: mov     al, byte ptr ds:[417h]
                shr     al, 4
                and     al, 7
                out     60h, al
                mov     al, 0ffh
                push    edx
                mov     dx, 378h
                out     dx, al
                pop     edx
                pop     ecx
                pop     eax
                ret

public drawledhdscr_
drawledhdscr_:  cmp     byte [_config_enableleds], 0
                je      drawledhddont
                cmp     byte [_config_enableleds], 6
                je      drawledhddont
                push    ecx
                push    edx
                xor     ecx, ecx
                test    dword ptr [_ledbyte], 20h
                jz      drawledhdscr1
                mov     ecx, [_hdledcolor]
drawledhdscr1:  mov     edx, 350
                call    drawledscr
                pop     edx
                pop     ecx
drawledhddont:  ret

_TEXT ends

_DATA  segment dword public use32 'DATA'

include data\fellow.inc
include data\memory.inc
include data\floppy.inc
include data\joymouse.inc
include data\cia.inc
include data\inout.inc
include data\bus.inc
include data\graphem.inc

drawledkbd_tab  db      0, 0, 2, 2, 4, 4, 6, 6, 1, 1, 3, 3, 5, 5, 7, 7  ; KB1
                db      0, 0, 2, 2, 4, 4, 6, 6, 1, 1, 3, 3, 5, 5, 7, 7
                db      0, 2, 4, 6, 1, 3, 5, 7, 0, 2, 4, 6, 1, 3, 5, 7  ; KB2
                db      0, 2, 4, 6, 1, 3, 5, 7, 0, 2, 4, 6, 1, 3, 5, 7
                db      0, 2, 4, 6, 4, 6, 4, 6, 1, 3, 5, 7, 5, 7, 5, 7  ; KB3
                db      1, 3, 5, 7, 5, 7, 5, 7, 1, 3, 5, 7, 5, 7, 5, 7
drawledkbd_kbd  db      0, 1, 2, 3, 4, 5, 6, 7  ; NCS
                db      0, 4, 2, 6, 1, 5, 3, 7  ; NSC
                db      0, 1, 4, 5, 2, 3, 6, 7  ; CNS
                db      0, 2, 4, 6, 1, 3, 5, 7  ; CSN
                db      0, 2, 1, 3, 4, 6, 5, 7  ; SCN
                db      0, 4, 1, 5, 2, 6, 3, 7  ; SNC



_oldmotorstate:  dd      0

_DATA  ends

_BSS  segment dword public use32 'BSS'
_BSS  ends

end
